#ifndef OBJ_LOADER_H
#define OBJ_LOADER_H

#include <QString>

class OBJData;
class OBJGroup;

class OBJLoader
{
public:

	static OBJData* loadFile( QString filename );

private:

	// parses a position line (starting with "v")
	// returns false on an error
	static bool parsePosition( int lineNumber, QString line,
		QStringList tokens, OBJData* pOBJData );

	// parses a texture coordinate line (starting with "vt")
	// returns false on an error
	static bool parseTextureCoordinate( int lineNumber, QString line,
		QStringList tokens, OBJData* pOBJData );

	// parses a normal line (starting with "vn")
	// returns false on an error
	static bool parseNormal( int lineNumber, QString line,
		QStringList tokens, OBJData* pOBJData );

	// parses a face line (starting with "f" or "fo")
	// returns false on an error
	static bool parseFace( int lineNumber, QString line,
		QStringList tokens, OBJGroup* pCurrentGroup );

	// given the tokens in a face line
	// returns if the attached attributes are consistent
	static bool isFaceLineAttributesConsistent( QStringList tokens,
		bool* pHasTextureCoordinates, bool* pHasNormals );

	// objFaceVertexToken is something of the form:
	// "int"
	// "int/int"
	// "int/int/int", 
	// "int//int"
	// i.e. one of the delimited int strings that specify a vertex and its attributes
	// 
	// returns:
	// whether the vertex is valid
	// and the indices in the out parameters
	static bool getVertexAttributes( QString objFaceVertexToken,
		int* pPositionIndex, int* pTextureCoordinateIndex, int* pNormalIndex );
};



#if 0

#include <vector>

#include <vecmath/Vector2f.h>
#include <vecmath/Vector3f.h>
#include "../geometry/IndexedFace.h"

class OBJLoader
{
public:

	static const char* LINE_DELIMITERS;
	static const char* FACE_DELIMITERS;

	// Load an Alias|Wavefront OBJ file
	// TODO: do groups eventually, define a group, etc
	static bool loadOBJFile( const char* filename,
		std::vector< Vector3f >* pvPositions,
		std::vector< Vector2f >* pvTextureCoordinates,
		std::vector< Vector3f >* pvNormals,
		std::vector< IndexedFace >* pvIndexedFaces );

private:

	static bool parseLine( char* szLine, int iLineNum,
		std::vector< Vector3f >* pvPositions,
		std::vector< Vector2f >* pvTextureCoordinates,
		std::vector< Vector3f >* pvNormals,		
		std::vector< IndexedFace >* pvIndexedFaces );

	static bool parseVertex( int iLineNum,
		std::vector< char* >* pvTokens,
		std::vector< Vector3f >* pvPositions );

	static bool parseTextureCoordinate( int iLineNum,
		std::vector< char* >* pvTokens,
		std::vector< Vector2f >* pvTextureCoordinates );

	static bool parseNormal( int iLineNum,
		std::vector< char* >* pvTokens,
		std::vector< Vector3f >* pvNormals );

	static bool parseFace( int iLineNum,
		std::vector< char* >* pvTokens,
		std::vector< IndexedFace >* pvIndexedFaces );
};

#endif

#endif // OBJ_LOADER_H
