#ifndef MATH_UTILS_H
#define MATH_UTILS_H

#include "common/BasicTypes.h"

class MathUtils
{
public:

	static const double E;
	static const double PI;

	static float degreesToRadians( float degrees );
	static float radiansToDegrees( float radians );	

	// clamps x to between min (inclusive) and max (exclusive)
	static int clampToRangeInt( int x, int min, int max );
	static float clampToRangeFloat( float x, float min, float max );
	static double clampToRangeDouble( double x, double min, double max );

	// TODO: rename these linearRemap

	static float rescaleFloatToFloat( float value,
		float inputMin, float inputMax,
		float outputMin, float outputMax );

	static int rescaleFloatToInt( float value,
		float fMin, float fMax,
		int iMin, int iMax );

	static float rescaleIntToFloat( int value,
		int iMin, int iMax,
		float fMin, float fMax );

	static int rescaleIntToInt( int value,
		int inMin, int inMax,
		int outMin, int outMax );

	// returns x * ( 1 - f ) + y * f;
	static float lerp( float x, float y, float f );
	
	static double lerp( double x, double y, double f );

	static float distanceSquared( float x0, float y0, float x1, float y1 );

	static float gaussianWeight( float r, float sigma );

private:

};

#endif
