#ifndef QUAT4F_H
#define QUAT4F_H

class Vector3f;
class Vector4f;

class Quat4f
{
public:

	Quat4f();
	// q = w + x * i + y * j + z * k
	Quat4f( float w, float x, float y, float z );
		
	Quat4f( const Quat4f& rq ); // copy constructor
	Quat4f& operator = ( const Quat4f& rq ); // assignment operator
	// no destructor necessary

	// returns a quaternion with 0 real part
	Quat4f( const Vector3f& v );

	// copies the components of a Vector4f directly into this quaternion
	Quat4f( const Vector4f& v );

	// returns the ith element (mod 4)
	const float& operator [] ( int i ) const;
	float& operator [] ( int i );

	float w() const;
	float x() const;
	float y() const;
	float z() const;
	Vector3f xyz() const;

	float abs() const;
	float absSquared() const;
	void normalize();
	Quat4f normalized() const;

	void conjugate();
	Quat4f conjugated() const;

	void invert();
	Quat4f inverse() const;

	void getAxisAngle( float* pfRadians, float* pfx, float* pfy, float* pfz ); // returns unit vector for rotation and radians about the unit vector
	void getAxisAngle( float* pfRadians, Vector3f& rvAxis );

	// sets this quaternion to be a rotation of fRadians about v = < fx, fy, fz >, v need not necessarily be unit length
	void setAxisAngle( float fRadians, float fx, float fy, float fz );
	void setAxisAngle( float fRadians, const Vector3f& crAxis );

	Vector3f rotateVector( const Vector3f& v );

	// ---- Utility ----
	void print();
 
	 // quaternion dot product (a la vector)
	static float dot( const Quat4f& q0, const Quat4f& q1 );	
	
	// linear (stupid) interpolation
	static Quat4f lerp( const Quat4f& q0, const Quat4f& q1, float alpha );

	// TODO: http://number-none.com/product/Understanding%20Slerp,%20Then%20Not%20Using%20It/
	// look at the other ones - nlerp and log-quaternion lerp

	// TODO: epsilon
	static Quat4f slerp( const Quat4f& q0, const Quat4f& q1, float alpha, float cosOmegaThreshold = 0.9995 );
	
	// returns a unit quaternion that's a uniformly distributed rotation
	// given u[i] is a uniformly distributed random number in [0,1]
	// taken from Graphics Gems II
	static Quat4f randomRotation( float u0, float u1, float u2 );

#if 0
	// rotates pvVector by the rotation in pqRotation
	static void rotateVector( Quat4f* pqRotation, Vector3f* pvVector, Vector3f* pvOut );
#endif

private:

	float m_afElements[4];

};

Quat4f operator + ( const Quat4f& q0, const Quat4f& q1 );
Quat4f operator - ( const Quat4f& q0, const Quat4f& q1 );
Quat4f operator * ( const Quat4f& q0, const Quat4f& q1 );
Quat4f operator * ( float f, const Quat4f& q );
Quat4f operator * ( const Quat4f& q, float f );

#endif
