#ifndef VECTOR_3F_H
#define VECTOR_3F_H

class Vector2f;
class Vector3d;
class Vector3i;

class Vector3f
{
public:

	Vector3f();
	Vector3f( float x, float y, float z );

	Vector3f( const Vector2f& xy, float z );
	Vector3f( float x, const Vector2f& yz );

	// copy constructors
	Vector3f( const Vector3f& rv );
	Vector3f( const Vector3d& rv );
	Vector3f( const Vector3i& rv );

	// assignment operators
	Vector3f& operator = ( const Vector3f& rv );
	Vector3f& operator = ( const Vector3d& rv );
	Vector3f& operator = ( const Vector3i& rv );

	// no destructor necessary

	// returns the ith element (mod 3)
	const float& operator [] ( int i ) const;
	float& operator [] ( int i );

	// TODO: const and const ref versions

	float x() const;
	float y() const;
	float z() const;

	Vector2f xy() const;
	Vector2f xz() const;
	Vector2f yz() const;
	// TODO: all the other combinations

	Vector3f xyz() const;
	Vector3f yzx() const;
	Vector3f zxy() const;
	// TODO: all the other combinations

	float abs() const;
	float absSquared() const;
	void normalize();
	Vector3f normalized() const;

	void negate();

	// ---- Utility ----
	operator const float* (); // automatic type conversion for GL
	void print() const;

	static float dot( const Vector3f& v0, const Vector3f& v1 );	

	static Vector3f cross( const Vector3f& v0, const Vector3f& v1 );

	// returns v0 * ( 1 - alpha ) * v1 * alpha
	static Vector3f lerp( const Vector3f& v0, const Vector3f& v1, float alpha );

private:

	float m_afElements[3];

};

Vector3f operator + ( const Vector3f& v0, const Vector3f& v1 );
Vector3f operator - ( const Vector3f& v0, const Vector3f& v1 );
Vector3f operator * ( const Vector3f& v0, const Vector3f& v1 );
Vector3f operator / ( const Vector3f& v0, const Vector3f& v1 );

Vector3f operator - ( const Vector3f& v );
Vector3f operator * ( float f, const Vector3f& v );
Vector3f operator * ( const Vector3f& v, float f );

#endif // VECTOR_3F_H
