#ifndef FFMPEG_VIDEO_H
#define FFMPEG_VIDEO_H

#include <QString>

#include "common/Reference.h"
#include "common/ReferenceCountedArray.h"
#include "video/IVideo.h"

// stdint.h requires __STDC_CONSTANT_MACROS to be defined
// in order to define INT64_C()
// which is needed by ffmpeg
#define __STDC_CONSTANT_MACROS

#include <ffmpeg/avcodec.h>
#include <ffmpeg/avformat.h>
#include <ffmpeg/swscale.h>

#undef __STDC_CONSTANT_MACROS


// A video loaded using ffmpeg implementing the IVideo interface
class FFMPEGVideo : public IVideo
{
public:

	static Reference< IVideo > fromFile( QString filename );
	virtual ~FFMPEGVideo();

	virtual int64 numFrames();
	virtual int framePeriodMilliseconds();
	virtual float framePeriodSeconds();

	virtual int width();
	virtual int height();
	virtual int bytesPerFrame();

	virtual int64 getNextFrameIndex();
	virtual void setNextFrameIndex( int64 frameIndex );

	// reads the next frame and automatically advances to the next frame index
	virtual bool getNextFrame( UnsignedByteArray dataOut );

private:

	FFMPEGVideo( AVFormatContext* pFormatContext, int iVideoStreamIndex,
		AVCodecContext* pCodecContext,
		AVFrame* pFrameRaw,
		AVFrame* pFrameRGB,
		SwsContext* pSWSContext );

	// sets the underlying video's next frame to seekTarget
	bool seek( int64 seekTarget );

	// reads the next frame and puts it into m_pFrameRaw	
	bool readNextFrameRaw();

	// decodes m_pFrameRaw into RGB
	// the data is stored in m_pFrameRGB (which must be valid)
	void convertRawFrameToRGB();

	// initially false
	// set to true once global ffmpeg initialization is complete
	// (initialized the first time an FFMPEGVideo is created)
	static bool s_bInitialized;

	AVFormatContext* m_pFormatContext;
	int m_iVideoStreamIndex;
    AVCodecContext* m_pCodecContext;
	AVFrame* m_pFrameRaw;
	AVFrame* m_pFrameRGB;
	SwsContext* m_pSWSContext; // for YUV --> RGB conversion

	int m_iWidth;
	int m_iHeight;

	bool m_bSeeking;
	int64 m_iNextFrameIndex;
};

#endif // FFMPEG_VIDEO_H
