#ifndef IMAGE_DIRECTORY_VIDEO_H
#define IMAGE_DIRECTORY_VIDEO_H

#include <QImage>
#include <QString>
#include <QStringList>

#include "common/Reference.h"
#include "common/ReferenceCountedArray.h"
#include "video/IVideo.h"

// A video from a directory of images
class ImageDirectoryVideo : public IVideo
{
public:

	static Reference< IVideo > fromDirectory( QString directoryName, int framePeriodMilliseconds );

	virtual int64 numFrames();
	virtual int framePeriodMilliseconds();
	virtual float framePeriodSeconds();

	virtual int width();
	virtual int height();
	virtual int bytesPerFrame();

	virtual int64 getNextFrameIndex();
	virtual void setNextFrameIndex( int64 frameIndex );

	virtual bool getNextFrame( UnsignedByteArray dataOut );

private:

	ImageDirectoryVideo( QString directoryName, QStringList filenames, int framePeriodMilliseconds );

	QString m_qsDirectoryName;
	QStringList m_qslFilenames;
	int m_iFramePeriodMilliseconds;

	int m_iNextFrameIndex;
	QImage m_qiNextFrame;
	int m_iWidth;
	int m_iHeight;
};

#endif // IMAGE_DIRECTORY_VIDEO_H
