#include "Cg/CgProgramWrapper.h"

#include <cassert>

// static
CgProgramWrapper* CgProgramWrapper::create( QString filename, QString entryPoint, CGcontext context, CGprofile profile )
{
	CGprogram program = cgCreateProgramFromFile
	(
		context,
		CG_SOURCE,
		qPrintable( filename ),
		profile,
		qPrintable( entryPoint ),
		NULL
	);

	if( program != NULL )
	{
		return new CgProgramWrapper( program );
	}
	else
	{
#if _DEBUG
		fprintf( stderr, "%s\n", cgGetLastListing( context ) );
#endif
		return NULL;
	}
}

// virtual
CgProgramWrapper::~CgProgramWrapper()
{
	cgDestroyProgram( m_cgProgram );
}

bool CgProgramWrapper::isCompiled()
{
	return cgIsProgramCompiled( m_cgProgram );
}

void CgProgramWrapper::bind()
{
	cgGLBindProgram( m_cgProgram );
}

void CgProgramWrapper::load()
{
	cgGLLoadProgram( m_cgProgram );
}

bool CgProgramWrapper::isLoaded()
{
	return cgGLIsProgramLoaded( m_cgProgram );
}

// returns the loaded parameter
CGparameter CgProgramWrapper::getNamedParameter( QString name )
{
	// it doesn't have it, try to load it
	if( !( m_qhProgramParameters.contains( name ) ) )
	{
		CGparameter parameter = cgGetNamedProgramParameter
		(
			m_cgProgram,
			CG_PROGRAM,
			qPrintable( name )
		);
		assert( parameter != NULL );

		// if the parameter exists, then put it in the hash table for next time
		if( parameter != NULL )
		{
			m_qhProgramParameters[ name ] = parameter;
		}

		// return it anyway
		return parameter;
	}
	else // otherwise just return it
	{
		return m_qhProgramParameters.value( name );
	}
}

CGprogram CgProgramWrapper::getProgram()
{
	return m_cgProgram;
}

//////////////////////////////////////////////////////////////////////////
// Private
//////////////////////////////////////////////////////////////////////////

CgProgramWrapper::CgProgramWrapper( CGprogram program ) :

	m_cgProgram( program )

{
	load();
}
