#include "GL/GLPerspectiveCamera.h"

#include <math/MathUtils.h>

//////////////////////////////////////////////////////////////////////////
// Public
//////////////////////////////////////////////////////////////////////////

GLPerspectiveCamera::GLPerspectiveCamera( const Vector3f& vEye,
										 const Vector3f& vCenter,
										 const Vector3f& vUp,
										 float fFovY, float fAspect,
										 float fZNear, float fZFar,
										 bool bIsInfinite )
{
	setPerspective( fFovY, fAspect, fZNear, fZFar, bIsInfinite );
	setLookAt( vEye, vCenter, vUp );
}

// virtual
GLPerspectiveCamera::~GLPerspectiveCamera()
{

}

void GLPerspectiveCamera::getPerspective( float* pfFovY, float* pfAspect,
										 float* pfZNear, float* pfZFar,
										 bool* pbZFarIsInfinite )
{
	*pfFovY = m_fFovY;
	*pfAspect = m_fAspect;

	*pfZNear = m_fZNear;
	*pfZFar = m_fZFar;

	if( pbZFarIsInfinite != NULL )
	{
		*pbZFarIsInfinite = m_bZFarIsInfinite;
	}
}

void GLPerspectiveCamera::setPerspective( float fFovY, float fAspect,
										 float fZNear, float fZFar,
										 bool bIsInfinite )
{
	m_fFovY = fFovY;
	m_fAspect = fAspect;

	// tan( theta / 2 ) = up / zNear
	float halfFovY = MathUtils::degreesToRadians( fFovY / 2.0f );
	float tanHalfFovY = tan( halfFovY );

	float top = fZNear * tanHalfFovY;
	float bottom = -top;

	// aspect = width / height = ( right - left ) / ( top - bottom )
	float right = fAspect * top;
	float left = -right;

	setFrustum( left, right, bottom, top, fZNear, fZFar, bIsInfinite );
}
