#include <cstdio>

#include "GL/GLUtilities.h"

//////////////////////////////////////////////////////////////////////////
// Public
//////////////////////////////////////////////////////////////////////////

// static
void GLUtilities::setupOrthoCamera( int viewportWidth, int viewportHeight )
{
	glViewport( 0, 0, viewportWidth, viewportHeight );

	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	gluOrtho2D( 0, viewportWidth, 0, viewportHeight );

	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();
}

// static
void GLUtilities::drawQuad( int width, int height, bool zeroOneTextureCoordinates, bool tTextureCoordinateOnBottom )
{
	drawQuad( 0, 0, width, height, zeroOneTextureCoordinates, tTextureCoordinateOnBottom );
}

// static
void GLUtilities::drawQuad( int x, int y, int width, int height, bool zeroOneTextureCoordinates, bool tTextureCoordinateOnBottom )
{
	float texCoords[8];
	float positions[8];

	if( zeroOneTextureCoordinates )
	{
		if( tTextureCoordinateOnBottom )
		{
			texCoords[0] = 0;
			texCoords[1] = 0;

			texCoords[2] = 1;
			texCoords[3] = 0;
			
			texCoords[4] = 1;
			texCoords[5] = 1;
			
			texCoords[6] = 0;
			texCoords[7] = 1;
		}
		else
		{
			texCoords[0] = 0;
			texCoords[1] = 1;

			texCoords[2] = 1;
			texCoords[3] = 1;

			texCoords[4] = 1;
			texCoords[5] = 0;

			texCoords[6] = 0;
			texCoords[7] = 0;
		}
	}
	else
	{
		if( tTextureCoordinateOnBottom )
		{
			texCoords[0] = 0;
			texCoords[1] = 0;

			texCoords[2] = width;
			texCoords[3] = 0;

			texCoords[4] = width;
			texCoords[5] = height;

			texCoords[6] = 0;
			texCoords[7] = height;
		}
		else
		{
			texCoords[0] = 0;
			texCoords[1] = height;

			texCoords[2] = width;
			texCoords[3] = height;

			texCoords[4] = width;
			texCoords[5] = 0;

			texCoords[6] = 0;
			texCoords[7] = 0;
		}
	}

	positions[0] = x;
	positions[1] = y;

	positions[2] = x + width;
	positions[3] = y;

	positions[4] = x + width;
	positions[5] = y + height;

	positions[6] = x;
	positions[7] = y + height;

	glBegin( GL_QUADS );

		for( int i = 0; i < 8; i += 2 )
		{
			glTexCoord2f( texCoords[i], texCoords[ i + 1 ] );
			glVertex2f( positions[i], positions[ i + 1 ] );
		}

	glEnd();
}

// static
float* GLUtilities::readDepthBuffer( int x, int y, int width, int height )
{
	float* depthBuffer = new float[ width * height ];
	glReadPixels( x, y, width, height, GL_DEPTH_COMPONENT, GL_FLOAT, depthBuffer );
	return depthBuffer;
}

// static
void GLUtilities::dumpDepthBufferText( int x, int y, int width, int height, const char* szFilename )
{
	float* depthBuffer = GLUtilities::readDepthBuffer( x, y, width, height );

	FILE* fp = fopen( szFilename, "w" );
	fprintf( fp, "Width = %d, Height = %d\n", width, height );

	int k = 0;
	for( int y = 0; y < height; ++y )
	{
		for( int x = 0; x < width; ++x )
		{
			fprintf( fp, "(%d, %d): %f\n", x, y, depthBuffer[k] );
			++k;
		}
	}
	fclose( fp );

	delete[] depthBuffer;
}

// static
void GLUtilities::dumpFrameBufferLuminanceBinary( int x, int y, int width, int height, const char* szFilename )
{
	int dimensions[2];
	dimensions[0] = width;
	dimensions[1] = height;

	float* pixels = new float[ width * height ];
	glReadPixels( x, y, width, height, GL_LUMINANCE, GL_FLOAT, pixels );
	
	FILE* fp = fopen( szFilename, "wb" );
	fwrite( dimensions, sizeof( int ), 2, fp );
	fwrite( pixels, sizeof( float ), width * height, fp );
	fclose( fp );

	delete[] pixels;
}

// static
void GLUtilities::dumpFrameBufferLuminanceText( int x, int y, int width, int height, const char* szFilename )
{
	float* pixels = new float[ width * height ];
	glReadPixels( x, y, width, height, GL_LUMINANCE, GL_FLOAT, pixels );

	FILE* fp = fopen( szFilename, "w" );
	fprintf( fp, "Width = %d, Height = %d\n", width, height );
	
	int k = 0;
	for( int y = 0; y < height; ++y )
	{
		for( int x = 0; x < width; ++x )
		{
			fprintf( fp, "(%d, %d): %f\n", x, y, pixels[k] );
			++k;
		}
	}
	fclose( fp );

	delete[] pixels;
}

// static
void GLUtilities::dumpFrameBufferRGBABinary( int x, int y, int width, int height, const char* szFilename )
{
	int dimensions[2];
	dimensions[0] = width;
	dimensions[1] = height;

	float* pixels = new float[ 4 * width * height ];
	glReadPixels( x, y, width, height, GL_RGBA, GL_FLOAT, pixels );

	FILE* fp = fopen( szFilename, "wb" );
	fwrite( dimensions, sizeof( int ), 2, fp );
	fwrite( pixels, sizeof( float ), 4 * width * height, fp );
	fclose( fp );

	delete[] pixels;
}

// static
void GLUtilities::dumpFrameBufferRGBAText( int x, int y, int width, int height, const char* szFilename )
{
	float* pixels = new float[ 4 * width * height ];
	glReadPixels( x, y, width, height, GL_RGBA, GL_FLOAT, pixels );

	int k = 0;
	FILE* fp = fopen( szFilename, "w" );

	for( int yy = 0; yy < height; ++yy )
	{
		for( int xx = 0; xx < width; ++xx )
		{
			fprintf( fp, "(%d, %d): (%f, %f, %f, %f)\n", xx + x, yy + y, pixels[k], pixels[k+1], pixels[k+2], pixels[k+3] );
			k += 4;
		}
	}

	fclose( fp );

	delete[] pixels;
}

// static
void GLUtilities::printGLRenderer()
{
	printf( "%s\n", glGetString( GL_RENDERER ) );
}

// static
void GLUtilities::printGLVendor()
{
	printf( "%s\n", glGetString( GL_VENDOR ) );
}

// static
void GLUtilities::printGLVersion()
{
	printf( "%s\n", glGetString( GL_VERSION ) );
}

// static
void GLUtilities::printGLLastError()
{
	GLenum glError = glGetError();
	switch( glError )
	{
	case GL_NO_ERROR:
		fprintf( stderr, "GL_NO_ERROR\n" );
		break;
	case GL_INVALID_ENUM:
		fprintf( stderr, "GL_INVALID_ENUM\n" );
		break;
	case GL_INVALID_VALUE:
		fprintf( stderr, "GL_INVALID_VALUE\n" );
		break;
	case GL_INVALID_OPERATION:
		fprintf( stderr, "GL_INVALID_OPERATION\n" );
		break;
	case GL_STACK_OVERFLOW:
		fprintf( stderr, "GL_STACK_OVERFLOW\n" );
		break;
	case GL_STACK_UNDERFLOW:
		fprintf( stderr, "GL_STACK_UNDERFLOW\n" );
		break;
	case GL_OUT_OF_MEMORY:
		fprintf( stderr, "GL_OUT_OF_MEMORY\n" );
		break;
	default:
		fprintf( stderr, "Unknown error!\n" );
	}
}
