#include <cmath>
#include <cstdlib>

#include "math/MathUtils.h"

#ifndef M_E
const double MathUtils::E = 2.71828182845904523536;
#else
const double MathUtils::E = M_E;
#endif
#ifndef M_PI
const double MathUtils::PI = 3.14159265358979323846;
#else
const double MathUtils::PI = M_PI;
#endif

// static
float MathUtils::degreesToRadians( float degrees )
{
	return( degrees * MathUtils::PI / 180.0f );
}

// static
float MathUtils::radiansToDegrees( float radians )
{
	return( radians * 180.0f / MathUtils::PI );
}

// static
int MathUtils::clampToRangeInt( int x, int min, int max )
{
	if( x >= max )
	{
		x = max - 1;
	}
	if( x < min )
	{
		x = min;
	}

	return x;
}

// static
float MathUtils::clampToRangeFloat( float x, float min, float max )
{
	if( x > max )
	{
		x = max;
	}
	if( x < min )
	{
		x = min;
	}
	
	return x;
}

// static
double MathUtils::clampToRangeDouble( double x, double min, double max )
{
	if( x > max )
	{
		x = max;
	}
	if( x < min )
	{
		x = min;
	}

	return x;
}


// static
float MathUtils::rescaleFloatToFloat( float value,
									 float inputMin, float inputMax,
									 float outputMin, float outputMax )
{
	float fraction = ( value - inputMin ) / ( inputMax - inputMin );
	return( outputMin + fraction * ( outputMax - outputMin ) );
}

// static
int MathUtils::rescaleFloatToInt( float value,
							 float fMin, float fMax,
							 int iMin, int iMax )
{
	float fraction = ( value - fMin ) / ( fMax - fMin );
	return( iMin + ( int )( fraction * ( iMax - iMin ) + 0.5f ) );
}

// static
float MathUtils::rescaleIntToFloat( int value,
							 int iMin, int iMax,
							 float fMin, float fMax )
{
	float fraction = ( ( float )( value - iMin ) ) / ( iMax - iMin );
	return( fMin + fraction * ( fMax - fMin ) );
}

// static
int MathUtils::rescaleIntToInt( int value,
							   int inMin, int inMax,
							   int outMin, int outMax )
{
	float fraction = ( ( float )( value - inMin ) ) / ( inMax - inMin );
	return( ( int )( ( outMin + fraction * ( outMax - outMin ) ) + 0.5f ) );
}

// static
float MathUtils::lerp( float x, float y, float f )
{
	return( x * ( 1.f - f ) + y * f );
}

// static
double MathUtils::lerp( double x, double y, double f )
{
	return( x * ( 1.0 - f ) + y * f );
}

// static
float MathUtils::distanceSquared( float x0, float y0, float x1, float y1 )
{
	float dx = x1 - x0;
	float dy = y1 - y0;

	return( dx * dx + dy * dy );
}

// static
float MathUtils::gaussianWeight( float r, float sigma )
{
	return exp( -( r * r ) / ( 2.f * sigma * sigma ) );
}
