#include "vecmath/GMatrixd.h"

GMatrixd::GMatrixd() :

	m_nRows( 0 ),
	m_nCols( 0 )
	// m_aData = NULL
{

}

GMatrixd::GMatrixd( uint nRows, uint nCols, double fillValue ) :

	m_nRows( nRows ),
	m_nCols( nCols ),
	m_aData( nRows * nCols, fillValue )

{
	
}

GMatrixd::GMatrixd( const GMatrixd& m ) :

	m_nRows( m.m_nRows ),
	m_nCols( m.m_nCols ),
	m_aData( m.m_aData.copy() )

{

}

// virtual
GMatrixd::~GMatrixd()
{
	// do nothing, releases data
}

uint GMatrixd::numRows() const
{
	return m_nRows;
}

uint GMatrixd::numCols() const
{
	return m_nCols;
}

uint GMatrixd::numElements() const
{
	return m_aData.length();
}

void GMatrixd::indexToSubscript( uint idx, uint* pi, uint* pj ) const
{
	*pi = idx / m_nRows;
	*pj = idx % m_nRows;
}

uint GMatrixd::subscriptToIndex( uint i, uint j ) const
{
	return( j * m_nRows + i );
}

void GMatrixd::resize( uint nRows, uint nCols )
{
	if( nRows == m_nRows && nCols == m_nCols )
	{
		return;
	}

	DoubleArray newData( nRows, nCols, 0 );
	if( nRows > m_nRows )
	{
		if( nCols > m_nCols )
		{
			// resized array is bigger: fill in original matrix
			for( uint i = 0; i < m_nRows; ++i )
			{
				for( uint j = 0; j < m_nCols; ++j )
				{
					newData[ j * nRows + i ] = m_aData[ j * m_nRows + i ];
				}
			}
		}
		else
		{
			// more rows, fewer columns
			for( uint i = 0; i < m_nRows; ++i )
			{
				for( uint j = 0; j < nCols; ++j )
				{
					newData[ j * nRows + i ] = m_aData[ j * m_nRows + i ];
				}
			}
		}
	}
	else
	{
		if( nCols > m_nCols )
		{
			// more columns, fewer rows
			for( uint i = 0; i < nRows; ++i )
			{
				for( uint j = 0; j < m_nCols; ++j )
				{
					newData[ j * nRows + i ] = m_aData[ j * m_nRows + i ];
				}
			}
		}
		else
		{
			// smaller in both dimensions
			for( uint i = 0; i < nRows; ++i )
			{
				for( uint j = 0; j < nCols; ++j )
				{
					newData[ j * nRows + i ] = m_aData[ j * m_nRows + i ];
				}
			}
		}
	}

	m_nRows = nRows;
	m_nCols = nCols;
	m_aData = newData;
}

void GMatrixd::reshape( uint nRows, uint nCols )
{
	assert( nRows * nCols == numElements() );

	m_nRows = nRows;
	m_nCols = nCols;
}

GMatrixd& GMatrixd::operator = ( const GMatrixd& m )
{
	if( &m != this )
	{
		m_nRows = m.m_nRows;
		m_nCols = m.m_nCols;
		m_aData = m.m_aData.copy();
	}
	return *this;
}

bool GMatrixd::isNull() const
{
	return( m_nRows == 0 || m_nCols == 0 );
}

void GMatrixd::fill( double d )
{
	for( int i = 0; i < m_aData.length(); ++i )
	{
		m_aData[ i ] = d;
	}	
}

double& GMatrixd::operator () ( uint i, uint j )
{
	return( m_aData[ j * m_nRows + i ] );
}

const double& GMatrixd::operator () ( uint i, uint j ) const
{
	return( m_aData[ j * m_nRows + i ] );
}

double& GMatrixd::operator [] ( uint k )
{
	return m_aData[ k ];
}

const double& GMatrixd::operator [] ( uint k ) const
{
	return m_aData[ k ];
}

DoubleArray GMatrixd::data() const
{
	return m_aData;
}
