#include <cmath>
#include <cstdio>
#include <cstdlib>

#include "vecmath/Vector3i.h"

#include "vecmath/Vector2i.h"
#include "vecmath/Vector3f.h"

//////////////////////////////////////////////////////////////////////////
// Public
//////////////////////////////////////////////////////////////////////////

Vector3i::Vector3i()
{
	m_aiElements[0] = 0;
	m_aiElements[1] = 0;
	m_aiElements[2] = 0;
}

Vector3i::Vector3i( int x, int y, int z )
{
	m_aiElements[0] = x;
	m_aiElements[1] = y;
	m_aiElements[2] = z;
}

Vector3i::Vector3i( const Vector2i& xy, int z )
{
	m_aiElements[0] = xy.x();
	m_aiElements[1] = xy.y();
	m_aiElements[2] = z;
}

Vector3i::Vector3i( int x, const Vector2i& yz )
{
	m_aiElements[0] = x;
	m_aiElements[1] = yz.x();
	m_aiElements[2] = yz.y();
}

Vector3i::Vector3i( const Vector3i& rv )
{
	m_aiElements[0] = rv.m_aiElements[0];
	m_aiElements[1] = rv.m_aiElements[1];
	m_aiElements[2] = rv.m_aiElements[2];
}

Vector3i& Vector3i::operator = ( const Vector3i& rv )
{
	if( this != &rv )
	{
		m_aiElements[0] = rv.m_aiElements[0];
		m_aiElements[1] = rv.m_aiElements[1];
		m_aiElements[2] = rv.m_aiElements[2];
	}
	return *this;
}

const int& Vector3i::operator [] ( int i ) const
{
	return m_aiElements[ i % 3 ];
}

int& Vector3i::operator [] ( int i )
{
	return m_aiElements[ i % 3 ];
}

int Vector3i::x() const
{
	return m_aiElements[0];
}

int Vector3i::y() const
{
	return m_aiElements[1];
}

int Vector3i::z() const
{
	return m_aiElements[2];
}

/*
Vector2f Vector3i::xy() const
{
	return Vector2f( m_aiElements[0], m_aiElements[1] );
}

Vector2f Vector3i::xz() const
{
	return Vector2f( m_aiElements[0], m_aiElements[2] );
}

Vector2f Vector3i::yz() const
{
	return Vector2f( m_aiElements[1], m_aiElements[2] );
}
*/

Vector3i Vector3i::xyz() const
{
	return Vector3i( m_aiElements[0], m_aiElements[1], m_aiElements[2] );
}

Vector3i Vector3i::yzx() const
{
	return Vector3i( m_aiElements[1], m_aiElements[2], m_aiElements[0] );
}

Vector3i Vector3i::zxy() const
{
	return Vector3i( m_aiElements[2], m_aiElements[0], m_aiElements[1] );
}

float Vector3i::abs() const
{
	return sqrt( static_cast< float >( absSquared() ) );
}

int Vector3i::absSquared() const
{
	return( m_aiElements[0] * m_aiElements[0] + m_aiElements[1] * m_aiElements[1] + m_aiElements[2] * m_aiElements[2] );
}

Vector3f Vector3i::normalized() const
{
	float length = abs();
	return Vector3f
	(
		m_aiElements[0] / length,
		m_aiElements[1] / length,
		m_aiElements[2] / length
	);
}

void Vector3i::negate()
{
	m_aiElements[0] = -m_aiElements[0];
	m_aiElements[1] = -m_aiElements[1];
	m_aiElements[2] = -m_aiElements[2];
}

// ---- Utility ----

Vector3i::operator const int* ()
{
	return m_aiElements;
}

void Vector3i::print() const
{
	printf( "< %5d, %5d, %5d >\n",
		m_aiElements[0], m_aiElements[1], m_aiElements[2] );
}

// static
int Vector3i::dot( const Vector3i& v0, const Vector3i& v1 )
{
	return v0.x() * v1.x() + v0.y() * v1.y() + v0.z() * v1.z();
}

// static
Vector3i Vector3i::cross( const Vector3i& v0, const Vector3i& v1 )
{
	return Vector3i
	(
		v0.y() * v1.z() - v0.z() * v1.y(),
		v0.z() * v1.x() - v0.x() * v1.z(),
		v0.x() * v1.y() - v0.y() * v1.x()
	);
}

// static
Vector3f Vector3i::lerp( const Vector3i& v0, const Vector3i& v1, float alpha )
{
	return alpha * ( v1 - v0 ) + Vector3f( v0 );
}

//////////////////////////////////////////////////////////////////////////
// Operators
//////////////////////////////////////////////////////////////////////////

Vector3i operator + ( const Vector3i& v0, const Vector3i& v1 )
{
	return Vector3i( v0.x() + v1.x(), v0.y() + v1.y(), v0.z() + v1.z() );
}

Vector3i operator - ( const Vector3i& v0, const Vector3i& v1 )
{
	return Vector3i( v0.x() - v1.x(), v0.y() - v1.y(), v0.z() - v1.z() );
}

Vector3i operator * ( const Vector3i& v0, const Vector3i& v1 )
{
	return Vector3i( v0.x() * v1.x(), v0.y() * v1.y(), v0.z() * v1.z() );
}

Vector3i operator - ( const Vector3i& v )
{
	return Vector3i( -v.x(), -v.y(), -v.z() );
}

Vector3i operator * ( int c, const Vector3i& v )
{
	return Vector3i( c * v.x(), c * v.y(), c * v.z() );
}

Vector3i operator * ( const Vector3i& v, int c )
{
	return Vector3i( c * v.x(), c * v.y(), c * v.z() );
}

Vector3f operator * ( float f, const Vector3i& v )
{
	return Vector3f( f * v.x(), f * v.y(), f * v.z() );
}

Vector3f operator * ( const Vector3i& v, float f )
{
	return Vector3f( f * v.x(), f * v.y(), f * v.z() );
}
